@@ -20,6 +20,7 @@ import threading
 import time
 import uuid
 from datetime import datetime, timedelta, timezone
+from unittest.mock import patch
 
 import pytest
 from dateutil.tz import tzlocal
@@ -28,15 +29,19 @@ from botocore import UNSIGNED
 from botocore.config import Config
 from botocore.credentials import (
     AssumeRoleProvider,
+    BotoProvider,
     CanonicalNameCredentialSourcer,
     ContainerProvider,
+    CredentialResolver,
     Credentials,
     DeferredRefreshableCredentials,
     EnvProvider,
     InstanceMetadataProvider,
     JSONFileCache,
+    ProcessProvider,
     ProfileProviderBuilder,
     ReadOnlyCredentials,
+    SSOProvider,
     create_credential_resolver,
 )
 from botocore.exceptions import (
@@ -47,9 +52,14 @@ from botocore.exceptions import (
 from botocore.session import Session
 from botocore.stub import Stubber
 from botocore.tokens import SSOTokenProvider
-from botocore.utils import datetime2timestamp
+from botocore.utils import (
+    ContainerMetadataFetcher,
+    InstanceMetadataFetcher,
+    datetime2timestamp,
+)
 from tests import (
     BaseEnvVar,
+    ClientHTTPStubber,
     IntegerRefresher,
     SessionHTTPStubber,
     StubbedSession,
@@ -58,6 +68,10 @@ from tests import (
     temporary_file,
     unittest,
 )
+from tests.functional.test_useragent import (
+    get_captured_ua_strings,
+    parse_registered_feature_ids,
+)
 
 TIME_IN_ONE_HOUR = datetime.now(tz=timezone.utc) + timedelta(hours=1)
 TIME_IN_SIX_MONTHS = datetime.now(tz=timezone.utc) + timedelta(hours=4320)
@@ -127,9 +141,8 @@ class TestCredentialRefreshRaces(unittest.TestCase):
         max_calls_allowed = math.ceil((end - start) / 2.0) + 1
         self.assertTrue(
             creds.refresh_counter <= max_calls_allowed,
-            "Too many cred refreshes, max: %s, actual: %s, "
-            "time_delta: %.4f"
-            % (max_calls_allowed, creds.refresh_counter, (end - start)),
+            f"Too many cred refreshes, max: {max_calls_allowed}, actual: "
+            f"{creds.refresh_counter}, time_delta: {end - start:.4f}",
         )
 
     def test_no_race_for_immediate_advisory_expiration(self):
@@ -206,9 +219,12 @@ class BaseAssumeRoleTest(BaseEnvVar):
 
     def create_random_credentials(self):
         return Credentials(
-            'fake-%s' % random_chars(15),
-            'fake-%s' % random_chars(35),
-            'fake-%s' % random_chars(45),
+            f'fake-{random_chars(15)}',
+            f'fake-{random_chars(35)}',
+            f'fake-{random_chars(45)}',
+            # The account_id gets resolved from the
+            # Arn in create_assume_role_response().
+            account_id='1234567890',
         )
 
     def assert_creds_equal(self, c1, c2):
@@ -241,9 +257,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
         credential_process = os.path.join(
             current_dir, 'utils', 'credentialprocess.py'
         )
-        self.credential_process = '{} {}'.format(
-            sys.executable, credential_process
-        )
+        self.credential_process = f'{sys.executable} {credential_process}'
 
     def mock_provider(self, provider_cls):
         mock_instance = mock.Mock(spec=provider_cls)
@@ -487,7 +501,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'source_profile = B\n'
             '[profile B]\n'
-            'credential_process = %s\n' % self.credential_process
+            f'credential_process = {self.credential_process}\n'
         )
         self.write_config(config)
 
@@ -520,7 +534,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'source_profile = B\n'
             '[profile B]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleB\n'
-            'web_identity_token_file = %s\n' % token_path
+            f'web_identity_token_file = {token_path}\n'
         )
         self.write_config(config)
 
@@ -561,7 +575,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'source_profile = B\n'
             '[profile B]\n'
-            'web_identity_token_file = %s\n' % token_path
+            f'web_identity_token_file = {token_path}\n'
         )
         self.write_config(config)
 
@@ -576,7 +590,9 @@ class TestAssumeRole(BaseAssumeRoleTest):
         token_cache_key = 'f395038c92f1828cbb3991d2d6152d326b895606'
         cached_token = {
             'accessToken': 'a.token',
-            'expiresAt': self.some_future_time(),
+            'expiresAt': self.some_future_time().strftime(
+                "%Y-%m-%dT%H:%M:%SZ"
+            ),
         }
         temp_cache = JSONFileCache(self.tempdir)
         temp_cache[token_cache_key] = cached_token
@@ -772,6 +788,26 @@ class TestAssumeRole(BaseAssumeRoleTest):
         self.assert_creds_equal(creds, expected_creds)
         self.assertEqual(self.actual_client_region, 'cn-north-1')
 
+    def test_assume_role_resolves_account_id(self):
+        config = (
+            '[profile A]\n'
+            'role_arn = arn:aws:iam::1234567890:role/RoleA\n'
+            'source_profile = B\n\n'
+            '[profile B]\n'
+            'aws_access_key_id = foo\n'
+            'aws_secret_access_key = bar\n'
+        )
+        self.write_config(config)
+        expected_creds = self.create_random_credentials()
+        response = self.create_assume_role_response(expected_creds)
+
+        session, stubber = self.create_session(profile='A')
+        stubber.add_response('assume_role', response)
+
+        actual_creds = session.get_credentials()
+        self.assert_creds_equal(actual_creds, expected_creds)
+        self.assertEqual(actual_creds.account_id, '1234567890')
+
 
 class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
     def setUp(self):
@@ -803,8 +839,8 @@ class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
             '[profile A]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'role_session_name = sname\n'
-            'web_identity_token_file = %s\n'
-        ) % self.token_file
+            f'web_identity_token_file = {self.token_file}\n'
+        )
         self.write_config(config)
         expected_params = {
             'RoleArn': 'arn:aws:iam::123456789:role/RoleA',
@@ -814,7 +850,7 @@ class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
         self.assert_session_credentials(expected_params, profile='A')
 
     def test_assume_role_env_vars(self):
-        config = '[profile B]\n' 'region = us-west-2\n'
+        config = '[profile B]\nregion = us-west-2\n'
         self.write_config(config)
         self.environ['AWS_ROLE_ARN'] = 'arn:aws:iam::123456789:role/RoleB'
         self.environ['AWS_WEB_IDENTITY_TOKEN_FILE'] = self.token_file
@@ -832,8 +868,8 @@ class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
             '[profile A]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'role_session_name = aname\n'
-            'web_identity_token_file = %s\n'
-        ) % self.token_file
+            f'web_identity_token_file = {self.token_file}\n'
+        )
         self.write_config(config)
 
         different_token = os.path.join(self.tempdir, str(uuid.uuid4()))
@@ -856,9 +892,7 @@ class TestProcessProvider(unittest.TestCase):
         credential_process = os.path.join(
             current_dir, 'utils', 'credentialprocess.py'
         )
-        self.credential_process = '{} {}'.format(
-            sys.executable, credential_process
-        )
+        self.credential_process = f'{sys.executable} {credential_process}'
         self.environ = os.environ.copy()
         self.environ_patch = mock.patch('os.environ', self.environ)
         self.environ_patch.start()
@@ -867,7 +901,7 @@ class TestProcessProvider(unittest.TestCase):
         self.environ_patch.stop()
 
     def test_credential_process(self):
-        config = '[profile processcreds]\n' 'credential_process = %s\n'
+        config = '[profile processcreds]\ncredential_process = %s\n'
         config = config % self.credential_process
         with temporary_file('w') as f:
             f.write(config)
@@ -880,8 +914,7 @@ class TestProcessProvider(unittest.TestCase):
 
     def test_credential_process_returns_error(self):
         config = (
-            '[profile processcreds]\n'
-            'credential_process = %s --raise-error\n'
+            '[profile processcreds]\ncredential_process = %s --raise-error\n'
         )
         config = config % self.credential_process
         with temporary_file('w') as f:
@@ -921,8 +954,8 @@ class TestSTSRegional(BaseAssumeRoleTest):
     def _get_assume_role_body(self, method_name):
         expiration = self.some_future_time()
         body = (
-            '<{method_name}Response>'
-            '  <{method_name}Result>'
+            f'<{method_name}Response>'
+            f'  <{method_name}Result>'
             '    <AssumedRoleUser>'
             '      <Arn>arn:aws:sts::0123456:user</Arn>'
             '      <AssumedRoleId>AKID:mysession-1567020004</AssumedRoleId>'
@@ -931,11 +964,11 @@ class TestSTSRegional(BaseAssumeRoleTest):
             '      <AccessKeyId>AccessKey</AccessKeyId>'
             '      <SecretAccessKey>SecretKey</SecretAccessKey>'
             '      <SessionToken>SessionToken</SessionToken>'
-            '      <Expiration>{expiration}</Expiration>'
+            f'      <Expiration>{expiration}</Expiration>'
             '    </Credentials>'
-            '  </{method_name}Result>'
-            '</{method_name}Response>'
-        ).format(method_name=method_name, expiration=expiration)
+            f'  </{method_name}Result>'
+            f'</{method_name}Response>'
+        )
         return body.encode('utf-8')
 
     def make_stubbed_client_call_to_region(self, session, stubber, region):
@@ -976,11 +1009,11 @@ class TestSTSRegional(BaseAssumeRoleTest):
             '[profile A]\n'
             'sts_regional_endpoints = regional\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
-            'web_identity_token_file = %s\n'
+            f'web_identity_token_file = {token_file}\n'
             'source_profile = B\n\n'
             '[profile B]\n'
             'aws_access_key_id = abc123\n'
-            'aws_secret_access_key = def456\n' % token_file
+            'aws_secret_access_key = def456\n'
         )
         self.write_config(config)
         # Make an arbitrary client and API call as we are really only
@@ -1164,3 +1197,477 @@ class TestContextCredentials(unittest.TestCase):
             s3.list_buckets()
             request = stubber.requests[0]
             assert self.ACCESS_KEY in str(request.headers.get('Authorization'))
+
+
+@pytest.mark.parametrize(
+    "creds_env_var,creds_file_content,patches,expected_feature_id",
+    [
+        (
+            'AWS_SHARED_CREDENTIALS_FILE',
+            '[default]\naws_access_key_id = FAKEACCESSKEY\naws_secret_access_key = FAKESECRET',
+            [
+                patch(
+                    "botocore.credentials.AssumeRoleProvider.load",
+                    return_value=None,
+                ),
+                patch(
+                    "botocore.credentials.EnvProvider.load", return_value=None
+                ),
+            ],
+            'n',
+        ),
+        (
+            'AWS_CONFIG_FILE',
+            '[default]\naws_access_key_id = FAKEACCESSKEY\naws_secret_access_key = FAKESECRET',
+            [
+                patch(
+                    "botocore.credentials.AssumeRoleProvider.load",
+                    return_value=None,
+                ),
+                patch(
+                    "botocore.credentials.EnvProvider.load", return_value=None
+                ),
+                patch(
+                    "botocore.credentials.SharedCredentialProvider.load",
+                    return_value=None,
+                ),
+            ],
+            'n',
+        ),
+    ],
+)
+def test_user_agent_has_file_based_feature_ids(
+    creds_env_var,
+    creds_file_content,
+    patches,
+    expected_feature_id,
+    tmp_path,
+    monkeypatch,
+):
+    credentials_file = tmp_path / "creds"
+    credentials_file.write_text(creds_file_content)
+    monkeypatch.setenv(creds_env_var, str(credentials_file))
+
+    for patch_obj in patches:
+        patch_obj.start()
+
+    try:
+        session = Session()
+        client = session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, expected_feature_id)
+    finally:
+        for patch_obj in patches:
+            patch_obj.stop()
+
+
+def _assert_feature_ids_in_ua(client, expected_feature_ids):
+    """Helper to test feature IDs appear in user agent for multiple calls."""
+    with ClientHTTPStubber(client, strict=True) as http_stubber:
+        http_stubber.add_response()
+        http_stubber.add_response()
+        client.list_buckets()
+        client.list_buckets()
+
+    ua_strings = get_captured_ua_strings(http_stubber)
+    for ua_string in ua_strings:
+        feature_list = parse_registered_feature_ids(ua_string)
+        for expected_id in expected_feature_ids:
+            assert expected_id in feature_list
+
+
+@pytest.mark.parametrize(
+    "config_content,env_vars,expected_source_features,expected_provider_feature",
+    [
+        # Test Case 1: Assume Role with source profile
+        (
+            '''[profile assume-role-test]
+role_arn = arn:aws:iam::123456789012:role/test-role
+source_profile = base
+
+[profile base]
+aws_access_key_id = FAKEACCESSKEY
+aws_secret_access_key = FAKESECRET''',
+            {},
+            [
+                'n',  # CREDENTIALS_PROFILE
+                'o',  # CREDENTIALS_PROFILE_SOURCE_PROFILE
+            ],
+            'i',  # CREDENTIALS_STS_ASSUME_ROLE
+        ),
+        # Test Case 2: Assume Role with named provider
+        (
+            '''[profile assume-role-test]
+role_arn = arn:aws:iam::123456789012:role/test-role
+credential_source = Environment''',
+            {
+                'AWS_ACCESS_KEY_ID': 'FAKEACCESSKEY',
+                'AWS_SECRET_ACCESS_KEY': 'FAKESECRET',
+            },
+            [
+                'g',  # CREDENTIALS_ENV_VARS
+                'p',  # CREDENTIALS_PROFILE_NAMED_PROVIDER
+            ],
+            'i',  # CREDENTIALS_STS_ASSUME_ROLE
+        ),
+    ],
+)
+def test_user_agent_has_assume_role_feature_ids(
+    config_content,
+    env_vars,
+    expected_source_features,
+    expected_provider_feature,
+    tmp_path,
+):
+    session = _create_assume_role_session(config_content, tmp_path)
+
+    # Set env vars if needed
+    with patch.dict(os.environ, env_vars, clear=True):
+        with SessionHTTPStubber(session) as stubber:
+            s3 = session.create_client('s3', region_name='us-east-1')
+            _add_assume_role_http_response(stubber, with_web_identity=False)
+            stubber.add_response()
+            stubber.add_response()
+            s3.list_buckets()
+            s3.list_buckets()
+
+    ua_strings = get_captured_ua_strings(stubber)
+    _assert_deferred_credential_feature_ids(
+        ua_strings, expected_source_features, expected_provider_feature
+    )
+
+
+@pytest.mark.parametrize(
+    "config_content,env_vars,expected_source_features,expected_provider_feature",
+    [
+        # Test Case 1: Assume Role with Web Identity through config profile
+        (
+            '''[profile assume-role-test]
+role_arn = arn:aws:iam::123456789012:role/test-role
+web_identity_token_file = {token_file}''',
+            {},
+            ['q'],  # CREDENTIALS_PROFILE_STS_WEB_ID_TOKEN
+            'k',  # CREDENTIALS_STS_ASSUME_ROLE_WEB_ID
+        ),
+        # Test Case 2: Assume Role with Web Identity through env vars
+        (
+            '',
+            {
+                'AWS_ROLE_ARN': 'arn:aws:iam::123456789012:role/test-role',
+                'AWS_WEB_IDENTITY_TOKEN_FILE': '{token_file}',
+                'AWS_ROLE_SESSION_NAME': 'test-session',
+            },
+            ['h'],  # CREDENTIALS_ENV_VARS_STS_WEB_ID_TOKEN
+            'k',  # CREDENTIALS_STS_ASSUME_ROLE_WEB_ID
+        ),
+    ],
+)
+def test_user_agent_has_assume_role_with_web_identity_feature_ids(
+    config_content,
+    env_vars,
+    expected_source_features,
+    expected_provider_feature,
+    tmp_path,
+):
+    token_file = tmp_path / 'token.jwt'
+    token_file.write_text('fake-jwt-token')
+    if 'AWS_WEB_IDENTITY_TOKEN_FILE' in env_vars:
+        env_vars['AWS_WEB_IDENTITY_TOKEN_FILE'] = str(token_file)
+    elif config_content and 'web_identity_token_file' in config_content:
+        config_content = config_content.replace(
+            '{token_file}', str(token_file)
+        )
+
+    session = _create_assume_role_session(config_content, tmp_path)
+
+    # Set env vars if needed
+    with patch.dict(os.environ, env_vars, clear=True):
+        with SessionHTTPStubber(session) as stubber:
+            s3 = session.create_client('s3', region_name='us-east-1')
+            _add_assume_role_http_response(stubber, with_web_identity=True)
+            stubber.add_response()
+            stubber.add_response()
+            s3.list_buckets()
+            s3.list_buckets()
+
+    ua_strings = get_captured_ua_strings(stubber)
+    _assert_deferred_credential_feature_ids(
+        ua_strings, expected_source_features, expected_provider_feature
+    )
+
+
+def _create_assume_role_session(config_content, tmp_path):
+    if config_content:
+        config_file = tmp_path / 'config'
+        config_file.write_text(config_content)
+        session = Session(profile='assume-role-test')
+        session.set_config_variable('config_file', str(config_file))
+    else:
+        session = Session()
+    return session
+
+
+def _add_assume_role_http_response(stubber, with_web_identity):
+    """Add HTTP response for AssumeRole or AssumeRoleWithWebIdentity call with proper credentials"""
+    expiration = (datetime.now(timezone.utc) + timedelta(hours=1)).strftime(
+        '%Y-%m-%dT%H:%M:%SZ'
+    )
+    method_name = (
+        'AssumeRoleWithWebIdentity' if with_web_identity else 'AssumeRole'
+    )
+    body = (
+        f'<{method_name}Response>'
+        f'  <{method_name}Result>'
+        '    <AssumedRoleUser>'
+        '      <Arn>arn:aws:sts::123456789012:user</Arn>'
+        '      <AssumedRoleId>AKID:test-session-123</AssumedRoleId>'
+        '    </AssumedRoleUser>'
+        '    <Credentials>'
+        f'      <AccessKeyId>FAKEASSUMEROLEKEY</AccessKeyId>'
+        f'      <SecretAccessKey>FAKEASSUMEROLSECRET</SecretAccessKey>'
+        '      <SessionToken>FAKETOKEN</SessionToken>'
+        f'      <Expiration>{expiration}</Expiration>'
+        '    </Credentials>'
+        f'  </{method_name}Result>'
+        f'</{method_name}Response>'
+    )
+    stubber.add_response(body=body.encode('utf-8'))
+
+
+def _assert_deferred_credential_feature_ids(
+    ua_strings,
+    expected_source_features,
+    expected_provider_feature,
+):
+    """Helper to assert feature IDs for deferred credential provider tests"""
+    assert len(ua_strings) == 3
+
+    # Request to fetch credentials should only register feature ids for the credential source
+    credential_source_feature_list = parse_registered_feature_ids(
+        ua_strings[0]
+    )
+    for feature in expected_source_features:
+        assert feature in credential_source_feature_list
+    assert expected_provider_feature not in credential_source_feature_list
+
+    # Original operation request should register feature ids for both the credential source and the provider
+    for i in [1, 2]:
+        operation_feature_list = parse_registered_feature_ids(ua_strings[i])
+        for feature in expected_source_features:
+            assert feature in operation_feature_list
+        assert expected_provider_feature in operation_feature_list
+
+
+class TestFeatureIdRegistered:
+    def test_user_agent_has_env_vars_credentials_feature_id(
+        self,
+        monkeypatch,
+        patched_session,
+    ):
+        env_vars = {
+            'AWS_ACCESS_KEY_ID': 'FAKEACCESSKEY',
+            'AWS_SECRET_ACCESS_KEY': 'FAKESECRET',
+            'AWS_SESSION_TOKEN': 'FAKETOKEN',
+        }
+        for var, value in env_vars.items():
+            monkeypatch.setenv(var, value)
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['g'])
+
+    def test_user_agent_has_code_credentials_feature_id(self, patched_session):
+        client = patched_session.create_client(
+            "s3",
+            region_name="us-east-1",
+            aws_access_key_id='FAKEACCESSKEY',
+            aws_secret_access_key='FAKESECRET',
+            aws_session_token='FAKETOKEN',
+        )
+        _assert_feature_ids_in_ua(client, ['e'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch(
+        "botocore.utils.InstanceMetadataFetcher.retrieve_iam_role_credentials"
+    )
+    def test_user_agent_has_imds_credentials_feature_id(
+        self,
+        mock_iam_fetcher,
+        mock_get_credentials,
+        patched_session,
+    ):
+        iam_role_fetcher = InstanceMetadataFetcher()
+        imds_provider = InstanceMetadataProvider(iam_role_fetcher)
+        credential_resolver = CredentialResolver([imds_provider])
+
+        fake_credentials = {
+            "role_name": "FAKEROLE",
+            "access_key": "FAKEACCESSKEY",
+            "secret_key": "FAKESECRET",
+            "token": "FAKETOKEN",
+            "expiry_time": "2099-01-01T00:00:00Z",
+        }
+        mock_iam_fetcher.return_value = fake_credentials
+        mock_get_credentials.return_value = credential_resolver
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['0'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch("botocore.credentials.ContainerMetadataFetcher.retrieve_full_uri")
+    def test_user_agent_has_container_credentials_feature_id(
+        self,
+        mock_container_metadata_fetcher,
+        mock_get_credentials,
+        monkeypatch,
+        patched_session,
+    ):
+        env_vars = {
+            'AWS_CONTAINER_CREDENTIALS_FULL_URI': 'http://localhost/foo',
+            'AWS_CONTAINER_AUTHORIZATION_TOKEN': 'Basic auth-token',
+        }
+        for var, value in env_vars.items():
+            monkeypatch.setenv(var, value)
+
+        container_metadata_fetcher = ContainerMetadataFetcher()
+        container_provider = ContainerProvider(
+            fetcher=container_metadata_fetcher
+        )
+        credential_resolver = CredentialResolver([container_provider])
+
+        fake_credentials = {
+            "AccessKeyId": "FAKEACCESSKEY",
+            "SecretAccessKey": "FAKESECRET",
+            "Token": "FAKETOKEN",
+            "Expiration": "2099-01-01T00:00:00Z",
+            "AccountId": "01234567890",
+        }
+
+        mock_container_metadata_fetcher.return_value = fake_credentials
+        mock_get_credentials.return_value = credential_resolver
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['z'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch("botocore.configloader.raw_config_parse")
+    def test_user_agent_has_boto2_config_credentials_feature_id(
+        self,
+        mock_boto2_config,
+        mock_get_credentials,
+        patched_session,
+    ):
+        boto_provider = BotoProvider()
+        credentials_resolver = CredentialResolver([boto_provider])
+
+        fake_credentials = {
+            "Credentials": {
+                "aws_access_key_id": "FAKEACCESSKEY",
+                "aws_secret_access_key": "FAKESECRETKEY",
+            }
+        }
+        mock_boto2_config.return_value = fake_credentials
+        mock_get_credentials.return_value = credentials_resolver
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['x'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch("botocore.credentials.ProcessProvider._retrieve_credentials_using")
+    @patch(
+        "botocore.credentials.ProcessProvider._credential_process",
+        return_value="Mock_credential_process",
+    )
+    def test_user_agent_has_process_provider_credentials_feature_id(
+        self,
+        _unused_mock_credentials_process,
+        mock_process_config,
+        mock_get_credentials,
+        patched_session,
+    ):
+        process_provider = ProcessProvider(None, None)
+        credentials_resolver = CredentialResolver([process_provider])
+
+        fake_credentials = {
+            'access_key': "FAKEACCESSKEY",
+            'secret_key': "FAKESECRETKEY",
+            'token': "FAKETOKEN",
+        }
+        mock_process_config.return_value = fake_credentials
+        mock_get_credentials.return_value = credentials_resolver
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['v', 'w'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch("botocore.credentials.CachedCredentialFetcher._load_from_cache")
+    @patch("botocore.credentials.SSOProvider._load_sso_config")
+    def test_user_agent_has_sso_legacy_credentials_feature_id(
+        self,
+        mock_load_sso_config,
+        mock_cached_credential_fetcher,
+        mock_get_credentials,
+        patched_session,
+    ):
+        sso_provider = SSOProvider(None, None, None)
+        credentials_resolver = CredentialResolver([sso_provider])
+        mock_get_credentials.return_value = credentials_resolver
+
+        fake_fetcher_kwargs = {
+            'sso_start_url': "https://test.awsapps.com/start",
+            'sso_region': "us-east-1",
+            'sso_role_name': "Administrator",
+            'sso_account_id': "1234567890",
+        }
+
+        fake_response = {
+            "ProviderType": "sso",
+            "Credentials": {
+                "role_name": "FAKEROLE",
+                "AccessKeyId": "FAKEACCESSKEY",
+                "SecretAccessKey": "FAKESECRET",
+                "SessionToken": "FAKETOKEN",
+                "Expiration": "2099-01-01T00:00:00Z",
+            },
+        }
+        mock_cached_credential_fetcher.return_value = fake_response
+        mock_load_sso_config.return_value = fake_fetcher_kwargs
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['t', 'u'])
+
+    @patch("botocore.credentials.create_credential_resolver")
+    @patch("botocore.credentials.CachedCredentialFetcher._load_from_cache")
+    @patch("botocore.credentials.SSOProvider._load_sso_config")
+    def test_user_agent_has_sso_credentials_feature_id(
+        self,
+        mock_load_sso_config,
+        mock_cached_credential_fetcher,
+        mock_get_credentials,
+        patched_session,
+    ):
+        sso_provider = SSOProvider(None, None, None)
+        credentials_resolver = CredentialResolver([sso_provider])
+        mock_get_credentials.return_value = credentials_resolver
+
+        fake_fetcher_kwargs = {
+            'sso_session': 'sample_test',  # Key difference from legacy SSO
+            'sso_start_url': "https://test.awsapps.com/start",
+            'sso_region': "us-east-1",
+            'sso_role_name': "Administrator",
+            'sso_account_id': "1234567890",
+        }
+
+        fake_response = {
+            "ProviderType": "sso",
+            "Credentials": {
+                "role_name": "FAKEROLE",
+                "AccessKeyId": "FAKEACCESSKEY",
+                "SecretAccessKey": "FAKESECRET",
+                "SessionToken": "FAKETOKEN",
+                "Expiration": "2099-01-01T00:00:00Z",
+            },
+        }
+        mock_cached_credential_fetcher.return_value = fake_response
+        mock_load_sso_config.return_value = fake_fetcher_kwargs
+
+        client = patched_session.create_client("s3", region_name="us-east-1")
+        _assert_feature_ids_in_ua(client, ['r', 's'])
